@@ -71,7 +71,9 @@
 #include <interfaces/launchconfigurationtype.h>
 
 #include "disassemblewidget.h"
+#ifdef KDEV_ENABLE_GDB_ATTACH_DIALOG
 #include "processselection.h"
+#endif
 #include "memviewdlg.h"
 #include "gdbparser.h"
 #include "gdboutputwidget.h"
@@ -115,7 +117,7 @@ public:
 
   virtual void viewCreated(Sublime::View* view)
   {
-      if (view->widget()->metaObject()->indexOfSignal(SIGNAL(requestRaise())) != -1)
+      if (view->widget()->metaObject()->indexOfSignal(QMetaObject::normalizedSignature("requestRaise()")) != -1)
           QObject::connect(view->widget(), SIGNAL(requestRaise()), view, SLOT(requestRaise()));
   }
 
@@ -138,6 +140,8 @@ CppDebuggerPlugin::CppDebuggerPlugin( QObject *parent, const QVariantList & ) :
     m_config(KGlobal::config(), "GDB Debugger"), m_session(0)
 {
     KDEV_USE_EXTENSION_INTERFACE( KDevelop::IStatus )
+    
+    core()->debugController()->initializeUi();
 
     setXMLFile("kdevgdbui.rc");
 
@@ -178,8 +182,8 @@ CppDebuggerPlugin::CppDebuggerPlugin( QObject *parent, const QVariantList & ) :
     // is undesirable. Besides, this makes tracepoint look even more similar
     // to printf debugging.
 // PORTING TODO broken - need intermediate signal?
-//     connect( gdbBreakpointWidget,   SIGNAL(tracingOutput(const QByteArray&)),
-//              procLineMaker,         SLOT(slotReceivedStdout(const QByteArray&)));
+//     connect( gdbBreakpointWidget,   SIGNAL(tracingOutput(QByteArray)),
+//              procLineMaker,         SLOT(slotReceivedStdout(QByteArray)));
 }
 
 void CppDebuggerPlugin::unload()
@@ -217,12 +221,13 @@ void CppDebuggerPlugin::setupActions()
     connect(action, SIGNAL(triggered(bool)), this, SLOT(slotExamineCore()));
     ac->addAction("debug_core", action);
 
-
+    #ifdef KDEV_ENABLE_GDB_ATTACH_DIALOG
     action = new KAction(KIcon("connect_creating"), i18n("Attach to Process"), this);
     action->setToolTip( i18n("Attach to process...") );
     action->setWhatsThis(i18n("<b>Attach to process</b><p>Attaches the debugger to a running process.</p>"));
     connect(action, SIGNAL(triggered(bool)), this, SLOT(slotAttachProcess()));
     ac->addAction("debug_attach", action);
+    #endif
 }
 
 void CppDebuggerPlugin::setupDBus()
@@ -355,8 +360,7 @@ DebugSession* CppDebuggerPlugin::createSession()
     connect(session, SIGNAL(showMessage(QString,int)), SLOT(controllerMessage(QString,int)));
     connect(session, SIGNAL(reset()), SIGNAL(reset()));
     connect(session, SIGNAL(finished()), SLOT(slotFinished()));
-    connect(session, SIGNAL(raiseOutputViews()), SIGNAL(raiseOutputViews()));
-    connect(session, SIGNAL(raiseVariableViews()), SIGNAL(raiseVariableViews()));
+    connect(session, SIGNAL(raiseGdbConsoleViews()), SIGNAL(raiseGdbConsoleViews()));
     return session;
 }
 
@@ -380,7 +384,7 @@ void CppDebuggerPlugin::slotExamineCore()
     job->start();
 }
 
-
+#ifdef KDEV_ENABLE_GDB_ATTACH_DIALOG
 void CppDebuggerPlugin::slotAttachProcess()
 {
     emit showMessage(this, i18n("Choose a process to attach to..."), 1000);
@@ -396,6 +400,7 @@ void CppDebuggerPlugin::slotAttachProcess()
     else
         attachProcess(pid);
 }
+#endif
 
 void CppDebuggerPlugin::attachProcess(int pid)
 {
