@@ -81,9 +81,13 @@ private Q_SLOTS:
     void testRemoteDebugInsertBreakpointPickupOnlyOnce();
     void testBreakpointWithSpaceInPath();
     void testBreakpointDisabledOnStart();
+    void testCatchpoint();
 
 private:
-    void waitForState(GDBDebugger::DebugSession *session, KDevelop::IDebugSession::DebuggerState state, const char *file, int line);
+    void waitForState(GDBDebugger::DebugSession *session,
+                      KDevelop::IDebugSession::DebuggerState state,
+                      const char *file, int line,
+                      bool expectFail = false);
 };
 
 }
