@@ -269,14 +269,33 @@ void DeclarationBuilder::visitQPropertyDeclaration(QPropertyDeclarationAST* node
   m_pendingPropertyDeclarations.insert(currentContext(), qMakePair(decl, node));
 }
 
-void DeclarationBuilder::visitForRangeDeclaration(ForRangeDeclarationAst* node)
+void DeclarationBuilder::handleRangeBasedFor(ExpressionAST* container, ForRangeDeclarationAst* iterator)
 {
-  AbstractType::Ptr listType = lastType();
+  ContextBuilder::handleRangeBasedFor(container, iterator);
 
-  DefaultVisitor::visitForRangeDeclaration(node);
+  if (!container || !iterator) {
+    return;
+  }
 
-  if (lastTypeWasAuto() && listType && m_lastDeclaration) {
+  if (lastTypeWasAuto() && m_lastDeclaration) {
     // auto support for range-based for
+    AbstractType::Ptr listType;
+
+    {
+      DUChainReadLocker lock;
+      container->ducontext = currentContext();
+      Cpp::ExpressionParser parser;
+      Cpp::ExpressionEvaluationResult res = parser.evaluateType( container, editor()->parseSession() );
+      listType = res.type.abstractType();
+    }
+
+    if (!listType) {
+      // invalid type
+      DUChainWriteLocker lock;
+      m_lastDeclaration->setAbstractType(AbstractType::Ptr(0));
+      return;
+    }
+
     AbstractType::Ptr realListType = TypeUtils::realType(listType);
     // step 1: find type of elements in list
     AbstractType::Ptr elementType;
@@ -287,11 +306,24 @@ void DeclarationBuilder::visitForRangeDeclaration(ForRangeDeclarationAst* node)
       // case b: look for begin(listType) function using ADL
       DUChainReadLocker lock;
       OverloadResolutionHelper helper = OverloadResolutionHelper( DUContextPointer(currentContext()), TopDUContextPointer(topContext()) );
+      helper.setKnownParameters(OverloadResolver::ParameterList(listType, false));
+      // first try begin in current context
       static const QualifiedIdentifier begin("begin");
       helper.setFunctionNameForADL(begin);
-      helper.setKnownParameters(OverloadResolver::ParameterList(listType, false));
-      helper.setFunctions( currentContext()->findDeclarations(begin, CursorInRevision::invalid(), AbstractType::Ptr(), 0, DUContext::OnlyFunctions) );
+      helper.setFunctions( currentContext()->findDeclarations(begin, CursorInRevision::invalid(),
+                                                              AbstractType::Ptr(), 0,
+                                                              DUContext::OnlyFunctions) );
       ViableFunction func = helper.resolve();
+      if (!func.isValid()) {
+        // not valid, fall-back to std, it's an associated namespace,
+        // see also: spec, 6.5.4
+        static const QualifiedIdentifier stdBegin("::std::begin");
+        helper.setFunctionNameForADL(stdBegin);
+        helper.setFunctions( currentContext()->findDeclarations(stdBegin, CursorInRevision::invalid(),
+                                                                AbstractType::Ptr(), 0,
+                                                                DUContext::OnlyFunctions) );
+        func = helper.resolve();
+      }
       if (func.isValid()) {
         AbstractType::Ptr type = func.declaration()->type<FunctionType>()->returnType();
         // see spec: for-range-declaration = *__begin;
@@ -300,8 +332,8 @@ void DeclarationBuilder::visitForRangeDeclaration(ForRangeDeclarationAst* node)
     }
 
     // step 2: set last type, but keep const&
+    DUChainWriteLocker lock;
     if (elementType) {
-      DUChainWriteLocker lock;
       AbstractType::Ptr type = m_lastDeclaration->abstractType();
       elementType->setModifiers(type->modifiers());
       if (ReferenceType::Ptr ref = type.cast<ReferenceType>()) {
@@ -310,6 +342,9 @@ void DeclarationBuilder::visitForRangeDeclaration(ForRangeDeclarationAst* node)
         type = elementType;
       }
       m_lastDeclaration->setAbstractType(type);
+    } else {
+      // invalid type
+      m_lastDeclaration->setAbstractType(AbstractType::Ptr(0));
     }
   }
 }
@@ -922,7 +957,7 @@ struct TemplateTypeExchanger : public KDevelop::TypeExchanger {
     TopDUContext* m_top;
 };
 
-Cpp::InstantiationInformation DeclarationBuilder::createSpecializationInformation(Cpp::InstantiationInformation base, UnqualifiedNameAST* name, KDevelop::DUContext* templateContext) {
+Cpp::InstantiationInformation DeclarationBuilder::createSpecializationInformation(const Cpp::InstantiationInformation& base, UnqualifiedNameAST* name, KDevelop::DUContext* templateContext) {
     if(name->template_arguments || base.isValid()) 
     {
       //Append a scope part
@@ -978,7 +1013,7 @@ void DeclarationBuilder::visitEnumerator(EnumeratorAST* node)
   uint oldEndToken = node->end_token;
   node->end_token = node->id + 1;
 
-  Identifier id(editor()->parseSession()->token_stream->token(node->id).symbol());
+  Identifier id(editor()->parseSession()->token_stream->symbol(node->id));
   Declaration* decl = openNormalDeclaration(0, node, id);
 
   node->end_token = oldEndToken;
@@ -1132,32 +1167,6 @@ void DeclarationBuilder::visitClassSpecifier(ClassSpecifierAST *node)
               }
             }
           }
-
-          //Update instantiations in case of template forward-declarations
-//           SpecialTemplateDeclaration<ForwardDeclaration>* templateForward = dynamic_cast<SpecialTemplateDeclaration<ForwardDeclaration>* > (decl);
-//           SpecialTemplateDeclaration<Declaration>* currentTemplate = dynamic_cast<SpecialTemplateDeclaration<Declaration>* >  (currentDeclaration());
-//
-//           if( templateForward && currentTemplate )
-//           {
-//             //Change the types of all the forward-template instantiations
-//             TemplateDeclaration::InstantiationsHash instantiations = templateForward->instantiations();
-//
-//             for( TemplateDeclaration::InstantiationsHash::iterator it = instantiations.begin(); it != instantiations.end(); ++it )
-//             {
-//               Declaration* realInstance = currentTemplate->instantiate(it.key().args, ImportTrace());
-//               Declaration* forwardInstance = dynamic_cast<Declaration*>(*it);
-//               //Now change the type of forwardInstance so it matches the type of realInstance
-//               CppClassType::Ptr realClass = realInstance->type<CppClassType>();
-//               CppClassType::Ptr forwardClass = forwardInstance->type<CppClassType>();
-//
-//               if( realClass && forwardClass ) {
-//                 //Copy the class from real into the forward-declaration's instance
-//                 copyCppClass(realClass.data(), forwardClass.data());
-//               } else {
-//                 kDebug(9007) << "Bad types involved in formward-declaration";
-//               }
-//             }
-//           }//templateForward && currentTemplate
         }
       }
     }//foreach
@@ -1201,7 +1210,7 @@ void DeclarationBuilder::visitBaseSpecifier(BaseSpecifierAST *node) {
         instance.access = KDevelop::Declaration::Private;
 
       if( node->access_specifier ) {
-        int tk = editor()->parseSession()->token_stream->token(node->access_specifier).kind;
+        quint16 tk = editor()->parseSession()->token_stream->token(node->access_specifier).kind;
 
         switch( tk ) {
           case Token_private:
@@ -1315,7 +1324,7 @@ void DeclarationBuilder::visitAliasDeclaration(AliasDeclarationAST* node)
 
   if( compilingContexts() ) {
     PushValue<bool> setTypeDef(m_inTypedef, true);
-    Declaration* decl = openDeclaration<Declaration>(node->name, node->name);
+    openDeclaration<Declaration>(node->name, node->name);
     closeDeclaration();
   }
 }
@@ -1343,7 +1352,7 @@ void DeclarationBuilder::visitNamespaceAliasDefinition(NamespaceAliasDefinitionA
   if( compilingContexts() ) {
     RangeInRevision range = editor()->findRange(node->namespace_name);
     DUChainWriteLocker lock(DUChain::lock());
-    NamespaceAliasDeclaration* decl = openDeclarationReal<NamespaceAliasDeclaration>(0, 0, Identifier(editor()->parseSession()->token_stream->token(node->namespace_name).symbol()), false, false, &range);
+    NamespaceAliasDeclaration* decl = openDeclarationReal<NamespaceAliasDeclaration>(0, 0, Identifier(editor()->parseSession()->token_stream->symbol(node->namespace_name)), false, false, &range);
     {
       QualifiedIdentifier id;
       identifierForNode(node->alias_name, id);
