@@ -1156,12 +1156,16 @@ int pp::next_token (Stream& input)
       }
       else if (isNumber(ch))
       {
+        KDevelop::SimpleCursor numericBeginPosition = input.inputPosition().castToSimpleCursor();
+
         PreprocessedContents byteNumber;
         {
           Stream ns(&byteNumber);
           skip_number(input, ns);
         }
 
+        KDevelop::SimpleCursor numericEndPosition = input.inputPosition().castToSimpleCursor();
+
         QString number(QString::fromUtf8(stringFromContents(byteNumber)));
         int base = 10;
         if (number.startsWith("0x")) {
@@ -1170,14 +1174,44 @@ int pp::next_token (Stream& input)
           base = 8;
         }
 
-        if (number.endsWith('u', Qt::CaseInsensitive)) {
-          token_uvalue = number.toULong(0, base);
-          nextToken = TOKEN_UNUMBER;
+        uint countUnsigned = 0, countLong = 0, countLongLong = 0;
+        while (true) {
+          if (number.endsWith("u", Qt::CaseInsensitive)) {
+                  ++countUnsigned;
+            number.chop(1);
+          } else if (number.endsWith("ll", Qt::CaseInsensitive)) { // "ll" before "l"
+                  ++countLongLong;
+            number.chop(2);
+          } else if (number.endsWith("l", Qt::CaseInsensitive)) {
+                  ++countLong;
+            number.chop (1);
+          } else {
+            break;
+          }
+        }
 
+        if ((countUnsigned > 1) || ((countLong + countLongLong) > 1)) {
+          KDevelop::ProblemPointer problem(new KDevelop::Problem);
+          problem->setFinalLocation(KDevelop::DocumentRange(currentFileName(), KDevelop::SimpleRange(numericBeginPosition, numericEndPosition)));
+          problem->setDescription(i18n("Invalid suffix combination"));
+          problemEncountered(problem);
+        }
+
+        bool parsedOk = 0;
+        if (countUnsigned) {
+          token_uvalue = number.toULongLong (&parsedOk, base);
+          nextToken = TOKEN_UNUMBER;
         } else {
-          token_value = number.toLong(0, base);
+          token_value = number.toLongLong (&parsedOk, base);
           nextToken = TOKEN_NUMBER;
         }
+
+        if (!parsedOk) {
+          KDevelop::ProblemPointer problem(new KDevelop::Problem);
+          problem->setFinalLocation(KDevelop::DocumentRange(currentFileName(), KDevelop::SimpleRange(numericBeginPosition, numericEndPosition)));
+          problem->setDescription(i18n("Invalid numeric value"));
+          problemEncountered(problem);
+        }
       }
       else
       {
