@@ -87,6 +87,9 @@ private slots:
 
   void testMultiByteCStrings();
   void testMultiByteComments();
+  //BEGIN C99 support
+  void testDesignatedInitializers();
+  //END C99 support
 
   void testTernaryEmptyExpression();
 
@@ -134,10 +137,6 @@ protected:
    * dump @p node and print problems of @c control
    */
   void dump(AST* node);
-  /**
-   * Retrieve string representation of @p node.
-   */
-  QString stringForNode(AST* node) const;
 
   bool hasKind(AST*, AST::NODE_KIND);
   AST* getAST(AST*, AST::NODE_KIND, int num = 0);
