@@ -24,7 +24,6 @@
 static char const * const _S_token_names[] = {
   "K_DCOP",
   "Q_OBJECT",
-  "__attribute__",
   "__typeof",
   "and",
   "and_eq",
@@ -84,6 +83,7 @@ static char const * const _S_token_names[] = {
   "mutable",
   "namespace",
   "new",
+  "noexcept",
   "not",
   "not_eq",
   "nullptr",
@@ -142,7 +142,6 @@ static char const * const _S_token_names[] = {
 static char const * const _S_token_texts[] = {
   "K_DCOP",
   "Q_OBJECT",
-  "__attribute__",
   "__typeof",
   "&&",
   "&=",
@@ -202,6 +201,7 @@ static char const * const _S_token_texts[] = {
   "mutable",
   "namespace",
   "new",
+  "noexcept",
   "!",
   "!=",
   "nullptr",
