@@ -22,23 +22,22 @@
 
 #include "makejob.h"
 
-#include <QtCore/QStringList>
-#include <QtCore/QProcess>
 #include <QtCore/QFileInfo>
 
 #include <KDebug>
-#include <kshell.h>
-#include <kglobal.h>
-#include <klocale.h>
-#include <kprocess.h>
+#include <KShell>
+#include <KGlobal>
+#include <KProcess>
+
+#include <util/environmentgrouplist.h>
+#include <util/processlinemaker.h>
 
 #include <interfaces/icore.h>
 #include <interfaces/iplugincontroller.h>
-#include <util/environmentgrouplist.h>
+#include <interfaces/iproject.h>
+
 #include <project/projectmodel.h>
 #include <project/interfaces/ibuildsystemmanager.h>
-#include <util/processlinemaker.h>
-#include <interfaces/iproject.h>
 
 #include "makeoutputdelegate.h"
 #include "makeoutputmodel.h"
@@ -46,12 +45,15 @@
 
 using namespace KDevelop;
 
-MakeJob::MakeJob(MakeBuilder* builder, KDevelop::ProjectBaseItem* item, CommandType c,  const QString& overrideTarget )
+MakeJob::MakeJob(MakeBuilder* builder, KDevelop::ProjectBaseItem* item,
+                 CommandType c,  const QStringList& overrideTargets,
+                 const MakeVariables& variables )
     : OutputJob(builder)
     , m_builder(builder)
     , m_item(item)
     , m_command(c)
-    , m_overrideTarget(overrideTarget)
+    , m_overrideTargets(overrideTargets)
+    , m_variables(variables)
     , m_lineMaker(0)
     , m_process(0)
     , m_killed(false)
@@ -60,8 +62,8 @@ MakeJob::MakeJob(MakeBuilder* builder, KDevelop::ProjectBaseItem* item, CommandT
     setCapabilities(Killable);
 
     QString title;
-    if( !m_overrideTarget.isEmpty() )
-        title = i18n("Make: %1", m_overrideTarget);
+    if( !m_overrideTargets.isEmpty() )
+        title = i18n("Make: %1", m_overrideTargets.join(" "));
     else
         title = i18n("Make: %1", m_item->text());
 
@@ -81,7 +83,7 @@ MakeJob::~MakeJob()
 
 void MakeJob::start()
 {
-    kDebug(9037) << "Building with make" << m_command << m_overrideTarget;
+    kDebug(9037) << "Building with make" << m_command << m_overrideTargets.join(" ");
     if (!m_item)
     {
         setError(ItemNoLongerValidError);
@@ -115,7 +117,7 @@ void MakeJob::start()
     QStringList cmd = computeBuildCommand();
     if( cmd.isEmpty() ) {
         setError(BuildCommandError);
-        setErrorText(i18n("Could not create build command for target '%1'", m_overrideTarget));
+        setErrorText(i18n("Could not create build command for targets '%1'", m_overrideTargets.join(" ")));
         return emitResult();
     }
 
@@ -159,9 +161,9 @@ MakeJob::CommandType MakeJob::commandType()
     return m_command;
 }
 
-const QString & MakeJob::customTarget() const
+QStringList MakeJob::customTargets() const
 {
-    return m_overrideTarget;
+    return m_overrideTargets;
 }
 
 KUrl MakeJob::computeBuildDir(KDevelop::ProjectBaseItem* item) const
@@ -190,12 +192,6 @@ KUrl MakeJob::computeBuildDir(KDevelop::ProjectBaseItem* item) const
 QStringList MakeJob::computeBuildCommand() const
 {
     QStringList cmdline;
-//     QString cmdline = DomUtil::readEntry(dom, makeTool);
-//     int prio = DomUtil::readIntEntry(dom, priority);
-//     QString nice;
-//     if (prio != 0) {
-//         nice = QString("nice -n%1 ").arg(prio);
-//     }
 
     KSharedConfig::Ptr configPtr = m_item->project()->projectConfiguration();
     KConfigGroup builderGroup( configPtr, "MakeBuilder" );
@@ -211,17 +207,18 @@ QStringList MakeJob::computeBuildCommand() const
     {
         cmdline << "-k";
     }
-    
+
     int jobnumber = builderGroup.readEntry("Number Of Jobs", 1);
     if(jobnumber>1) {
         QString jobNumberArg = QString("-j%1").arg(jobnumber);
         cmdline << jobNumberArg;
     }
-    
+
     if( builderGroup.readEntry("Display Only", false) )
     {
         cmdline << "-n";
     }
+
     QString extraOptions = builderGroup.readEntry("Additional Options", QString());
     if( ! extraOptions.isEmpty() )
     {
@@ -229,32 +226,39 @@ QStringList MakeJob::computeBuildCommand() const
             cmdline << option;
     }
 
-    if( m_overrideTarget.isEmpty() )
+    MakeVariables::const_iterator it = m_variables.constBegin();
+    while ( it != m_variables.constEnd() )
+    {
+        cmdline += QString("%1=%2").arg(it->first).arg(it->second);
+        ++it;
+    }
+
+    if( m_overrideTargets.isEmpty() )
     {
         QString target;
         switch (m_item->type()) {
-        case KDevelop::ProjectBaseItem::Target:
-        case KDevelop::ProjectBaseItem::ExecutableTarget:
-        case KDevelop::ProjectBaseItem::LibraryTarget:
-            Q_ASSERT(m_item->target());
-            cmdline << m_item->target()->text();
-            break;
-        case KDevelop::ProjectBaseItem::BuildFolder:
-            target = builderGroup.readEntry("Default Target", QString());
-            if( !target.isEmpty() )
-                cmdline << target;
-            break;
-        default: break; }
+            case KDevelop::ProjectBaseItem::Target:
+            case KDevelop::ProjectBaseItem::ExecutableTarget:
+            case KDevelop::ProjectBaseItem::LibraryTarget:
+                Q_ASSERT(m_item->target());
+                cmdline << m_item->target()->text();
+                break;
+            case KDevelop::ProjectBaseItem::BuildFolder:
+                target = builderGroup.readEntry("Default Target", QString());
+                if( !target.isEmpty() )
+                    cmdline << target;
+                break;
+            default: break;
+        }
     }else
     {
-        cmdline << m_overrideTarget;
+        cmdline += m_overrideTargets;
     }
 
     bool runAsRoot = builderGroup.readEntry("Install As Root", false);
     if (runAsRoot && m_command == InstallCommand)
     {
         int suCommand = builderGroup.readEntry("Su Command", 0);
-        QStringList kdesuline;
         QStringList arguments;
         QString suCommandName;
         if (suCommand == 1) {
@@ -264,11 +268,10 @@ QStringList MakeJob::computeBuildCommand() const
           suCommandName = "sudo";
           arguments << cmdline;
         } else { //default
-          suCommandName = "kdesu";   
+          suCommandName = "kdesu";
           arguments << "-t" << "--" << cmdline;
         }
-        kdesuline << suCommandName << arguments;
-        cmdline = kdesuline;
+        cmdline = QStringList() << suCommandName << arguments;
     }
 
     return cmdline;
@@ -318,6 +321,7 @@ void MakeJob::procError( QProcess::ProcessError err )
     }
     emitResult();
 }
+
 void MakeJob::procFinished( int code, QProcess::ExitStatus status )
 {
     Q_UNUSED(code)
@@ -352,5 +356,4 @@ void MakeJob::setItem( ProjectBaseItem* item )
     m_item = item;
 }
 
-
 #include "makejob.moc"
