@@ -80,7 +80,6 @@
 #include <language/duchain/use.h>
 #include <interfaces/idocumentation.h>
 #include "cmakeprojectdata.h"
-#include <cmakeconfig.h>
 
 #include <language/highlighting/codehighlighting.h>
 #include <interfaces/iruncontroller.h>
@@ -471,14 +470,14 @@ CMakeManager::~CMakeManager()
 
 KUrl CMakeManager::buildDirectory(KDevelop::ProjectBaseItem *item) const
 {
+    CMakeFolderItem *fi=dynamic_cast<CMakeFolderItem*>(item);
     KUrl ret;
-    ProjectBaseItem* parent = item->parent();
+    ProjectBaseItem* parent = fi ? fi->formerParent() : item->parent();
     if (parent)
         ret=buildDirectory(parent);
     else
         ret=CMake::currentBuildDir(item->project());
     
-    CMakeFolderItem *fi=dynamic_cast<CMakeFolderItem*>(item);
     if(fi)
         ret.addPath(fi->buildDir());
     return ret;
@@ -578,6 +577,16 @@ KDevelop::ReferencedTopDUContext CMakeManager::initializeProject(KDevelop::IProj
 
 KDevelop::ProjectFolderItem* CMakeManager::import( KDevelop::IProject *project )
 {
+    kDebug(9042) << "== migrating cmake settings";
+    CMake::attemptMigrate(project);
+    kDebug(9042) << "== completed cmake migration";
+
+    kDebug(9042) << "== updating cmake settings from model";
+    int buildDirCount = CMake::buildDirCount(project);
+    for( int i = 0; i < buildDirCount; ++i )
+        CMake::updateConfig( project, i );
+    kDebug(9042) << "== completed updating cmake settings";
+
     CMakeFolderItem* m_rootItem=0;
     KUrl cmakeInfoFile(project->projectFileUrl());
     cmakeInfoFile = cmakeInfoFile.upUrl();
@@ -641,6 +650,7 @@ KDevelop::ProjectFolderItem* CMakeManager::import( KDevelop::IProject *project )
         w->setObjectName(project->name()+"_ProjectWatcher");
         w->addFile(cachefile.toLocalFile());
         connect(w, SIGNAL(dirty(QString)), this, SLOT(dirtyFile(QString)));
+        connect(w, SIGNAL(created(QString)), this, SLOT(dirtyFile(QString)));
         connect(w, SIGNAL(deleted(QString)), this, SLOT(deletedWatched(QString)));
         m_watchers[project] = w;
         Q_ASSERT(m_rootItem->rowCount()==0);
@@ -675,13 +685,39 @@ QSet<QString> filterFiles(const QStringList& orig)
     return ret;
 }
 
+QStringList resolvePaths(const KUrl& baseUrl, const QStringList& pathsToResolve) {
+    QStringList resolvedPaths;
+    foreach(const QString& pathToResolve, pathsToResolve)
+    {
+        QString dir(pathToResolve);
+        if(!pathToResolve.startsWith("#["))
+        {
+            if(KUrl( pathToResolve ).isRelative())
+            {
+                KUrl path(baseUrl);
+                path.addPath(pathToResolve);
+                dir=path.toLocalFile();
+            }
+
+            KUrl simp(dir); //We use this to simplify dir
+            simp.cleanPath();
+            dir=simp.toLocalFile();
+        }
+        resolvedPaths.append(dir);
+    }
+    return resolvedPaths;
+}
+
 QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolderItem* item )
 {
     Q_ASSERT(isReloading(item->project()));
     QList<KDevelop::ProjectFolderItem*> folderList;
     CMakeFolderItem* folder = dynamic_cast<CMakeFolderItem*>( item );
 
-    m_watchers[item->project()]->addDir(item->url().toLocalFile(), KDirWatch::WatchFiles);
+    {
+        QMutexLocker locker(&m_dirWatchersMutex);
+        m_watchers[item->project()]->addDir(item->url().toLocalFile(), KDirWatch::WatchFiles);
+    }
     
     KUrl cmakeListsPath(item->url());
     cmakeListsPath.addPath("CMakeLists.txt");
@@ -774,27 +810,8 @@ QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolder
 
         QStringList directories;
         directories += folder->url().toLocalFile(KUrl::RemoveTrailingSlash);
-
-        foreach(const QString& s, data.includeDirectories)
-        {
-            QString dir(s);
-            if(!s.startsWith("#["))
-            {
-                if(KUrl( s ).isRelative())
-                {
-                    KUrl path=folder->url();
-                    path.addPath(s);
-                    dir=path.toLocalFile();
-                }
-
-                KUrl simp(dir); //We use this to simplify dir
-                simp.cleanPath();
-                dir=simp.toLocalFile();
-            }
-
-            if(!directories.contains(dir))
-                directories.append(dir);
-        }
+        directories += resolvePaths(folder->url(), data.includeDirectories);
+        directories.removeDuplicates();
         folder->setIncludeDirectories(directories);
 //             kDebug(9042) << "setting include directories: " << folder->url() << directories << "result: " << folder->includeDirectories();
         folder->setDefinitions(data.definitions);
@@ -853,24 +870,39 @@ QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolder
                         break;
                 }
             
-            DefinesAttached* a = dynamic_cast<DefinesAttached*>(targetItem);
-            if(a)
-                a->defineVariables(targetProps["COMPILE_DEFINITIONS"]);
-            
-            DescriptorAttatched* datt=dynamic_cast<DescriptorAttatched*>(targetItem);
-            datt->setDescriptor(t.desc);
+            DefinesAttached* defAtt = dynamic_cast<DefinesAttached*>(targetItem);
+            if(defAtt)
+                defAtt->defineVariables(targetProps["COMPILE_DEFINITIONS"]);
 
+            DescriptorAttatched* descAtt=dynamic_cast<DescriptorAttatched*>(targetItem);
+            if(descAtt)
+                descAtt->setDescriptor(t.desc);
+
+            if(targetProps.contains("INCLUDE_DIRECTORIES")) {
+                IncludesAttached* incAtt = dynamic_cast<IncludesAttached*>(targetItem);
+                if(incAtt) {
+                    QStringList targetIncludeDirectories = resolvePaths(folder->url(), targetProps["INCLUDE_DIRECTORIES"]);
+                    targetIncludeDirectories.removeDuplicates();
+                    incAtt->setIncludeDirectories(targetIncludeDirectories);
+                }
+            }
+            
             KUrl::List tfiles;
             foreach( const QString & sFile, t.files)
             {
                 if(sFile.startsWith("#[") || sFile.isEmpty())
                     continue;
 
-                KUrl sourceFile(sFile);
-                if(sourceFile.isRelative()) {
+                KUrl sourceFile;
+                if(KUrl::isRelativeUrl(sFile)) {
                     sourceFile = folder->url();
                     sourceFile.addPath( sFile );
-                }
+                    if(!QFile::exists(sourceFile.toLocalFile())) {
+                        sourceFile = buildDirectory(folder);
+                        sourceFile.addPath( sFile );
+                    }
+                } else
+                    sourceFile = sFile;
                 sourceFile.cleanPath();
                 tfiles += sourceFile;
                 kDebug(9042) << "..........Adding:" << sourceFile << sFile << folder->url();
@@ -929,23 +961,20 @@ QList<KDevelop::ProjectTargetItem*> CMakeManager::targets() const
 
 KUrl::List CMakeManager::includeDirectories(KDevelop::ProjectBaseItem *item) const
 {
-    CMakeFolderItem* folder=0;
+    IncludesAttached* includer=0;
 //     kDebug(9042) << "Querying inc dirs for " << item;
-    while(!folder && item)
+    while(item)
     {
-        folder = dynamic_cast<CMakeFolderItem*>( item );
+        includer = dynamic_cast<IncludesAttached*>( item );
+        if(includer) {
+            QStringList dirs = includer->includeDirectories(item);
+            return resolveSystemDirs(item->project(), dirs);
+        }
         item = item->parent();
-//         kDebug(9042) << "Looking for a folder: " << (folder ? folder->url() : KUrl()) << item;
+//         kDebug(9042) << "Looking for an includer: " << item;
     }
-    if( !folder ) {
-        // Not a CMake folder, so no include-directories to be returned;
-        return KUrl::List();
-    }
-
-//     kDebug(9042) << "Include directories! -- before" << folder->includeDirectories();
-    KUrl::List l = resolveSystemDirs(folder->project(), folder->includeDirectories());
-//     kDebug(9042) << "Include directories!" << l;
-    return l;
+    // No includer found, so no include-directories to be returned;
+    return KUrl::List();
 }
 
 QHash< QString, QString > CMakeManager::defines(KDevelop::ProjectBaseItem *item ) const
@@ -971,7 +1000,7 @@ QHash< QString, QString > CMakeManager::defines(KDevelop::ProjectBaseItem *item
     return defs;
 }
 
-KDevelop::IProjectBuilder * CMakeManager::builder(KDevelop::ProjectFolderItem *) const
+KDevelop::IProjectBuilder * CMakeManager::builder() const
 {
     IPlugin* i = core()->pluginController()->pluginForExtension( "org.kdevelop.IProjectBuilder", "KDevCMakeBuilder");
     Q_ASSERT(i);
@@ -1036,6 +1065,7 @@ bool CMakeManager::reload(KDevelop::ProjectFolderItem* folder)
 
 void CMakeManager::reimport(CMakeFolderItem* fi)
 {
+    Q_ASSERT(fi && fi->project());
     Q_ASSERT(!isReloading(fi->project()));
     KJob *job=createImportJob(fi);
     job->setProperty("project", qVariantFromValue(fi->project()));
@@ -1262,7 +1292,10 @@ void CMakeManager::reloadFiles(ProjectFolderItem* item)
                 fileurl.adjustPath(KUrl::AddTrailingSlash);
                 ProjectFolderItem* it = new ProjectFolderItem( item->project(), fileurl, 0 );
                 reloadFiles(it);
-                m_watchers[item->project()]->addDir(fileurl.toLocalFile(), KDirWatch::WatchFiles);
+                {
+                    QMutexLocker locker(&m_dirWatchersMutex);
+                    m_watchers[item->project()]->addDir(fileurl.toLocalFile(), KDirWatch::WatchFiles);
+                }
                 newItems += it;
             }
         }
@@ -1699,6 +1732,8 @@ QPair<QString, QString> CMakeManager::cacheValue(KDevelop::IProject* project, co
 void CMakeManager::projectClosing(IProject* p)
 {
     m_projectsData.remove(p); 
+    
+    QMutexLocker locker(&m_dirWatchersMutex);
     m_watchers.remove(p);
 }
 
