@@ -80,7 +80,7 @@ public:
     virtual ~CMakeManager();
     virtual Features features() const { return Features(Folders | Targets | Files ); }
 //     virtual KDevelop::IProject* project() const;
-    virtual KDevelop::IProjectBuilder* builder(KDevelop::ProjectFolderItem*) const;
+    virtual KDevelop::IProjectBuilder* builder() const;
     virtual KUrl buildDirectory(KDevelop::ProjectBaseItem*) const;
     virtual KUrl::List includeDirectories(KDevelop::ProjectBaseItem *) const;
     virtual QHash<QString, QString> defines(KDevelop::ProjectBaseItem *) const;
@@ -146,6 +146,7 @@ private:
     
     QMutex m_reparsingMutex;
     QMutex m_busyProjectsMutex;
+    QMutex m_dirWatchersMutex;
     KDevelop::ReferencedTopDUContext initializeProject(KDevelop::IProject* project);
     
     KDevelop::ReferencedTopDUContext includeScript(const QString& file, KDevelop::IProject * project, const QString& currentDir,
