@@ -1003,7 +1003,7 @@ bool ExportLibraryDepsAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 }
 
 FileAst::FileAst()
-    : m_newlineConsume(false), m_noHexConversion(false)
+    : m_isFollowingSymlinks(false), m_newlineConsume(false), m_noHexConversion(false)
 {
 }
 
@@ -1264,6 +1264,8 @@ bool FindFileAst::parseFunctionInfo( const CMakeFunctionDesc& func )
             s=PATHS;
         else if(it->value=="PATH_SUFFIXES")
             s=PATH_SUFFIXES;
+        else if(it->value=="HINTS")
+            s=HINTS;
         else switch(s) {
             case NAMES:
                 m_filenames << it->value;
@@ -1281,7 +1283,7 @@ bool FindFileAst::parseFunctionInfo( const CMakeFunctionDesc& func )
                 break;
         }
     }
-    return !m_filenames.isEmpty() && !m_path.isEmpty();
+    return !m_filenames.isEmpty();
 }
 
 
@@ -1402,7 +1404,7 @@ bool FindPackageAst::parseFunctionInfo( const CMakeFunctionDesc& func )
     QList<CMakeFunctionArgument>::const_iterator itEnd=func.arguments.constEnd();
 
     bool ret=true;
-    enum State { None, Components };
+    enum State { None, Components, Paths };
     State s=None;
     for(; it!=itEnd; ++it)
     {
@@ -1413,8 +1415,11 @@ bool FindPackageAst::parseFunctionInfo( const CMakeFunctionDesc& func )
         else if(it->value=="NO_MODULE") m_noModule=true;
         else if(it->value=="REQUIRED") { m_isRequired=true; s=Components; }
         else if(it->value=="COMPONENTS") s=Components;
+        else if(it->value=="PATHS") s=Paths;
         else if(s==Components)
             m_components.append(it->value);
+        else if(s==Paths)
+            m_paths.append(it->value);
         else
             ret=false;
     }
@@ -1662,6 +1667,8 @@ bool GetCMakePropertyAst::parseFunctionInfo( const CMakeFunctionDesc& func )
         m_type=Commands;
     else if(type=="MACROS")
         m_type=Macros;
+    else if(type=="COMPONENTS")
+        m_type=Components;
     else
         return false;
     return true;
